-- 
-- BEAN'S DYNAMITE DERBY - TALLY
-- Graciously programmed by haya
--

-- Bonus types
local BDDTALLY_BONUS_NA = 0 
local BDDTALLY_BONUS_TIMELEFT = 1
local BDDTALLY_BONUS_POSITION = 2

-- States
local BDDTALLY_STATE_IGNORE = 0
local BDDTALLY_STATE_GOTTHRU_SLIDEIN = 1
local BDDTALLY_STATE_GOTTHRU_SLIDEUP = 2
local BDDTALLY_STATE_BOXES_SLIDEIN = 3
local BDDTALLY_STATE_TEXT_APPEAR = 4
local BDDTALLY_STATE_TEXT_PAUSE = 5
local BDDTALLY_STATE_RANK_APPEAR = 6
local BDDTALLY_STATE_RANK_PAUSE = 7
local BDDTALLY_STATE_DONE = 8

-- Grades
local BDDTALLY_RANK_LOSE = 1
local BDDTALLY_RANK_WINNER = 2
local BDDTALLY_RANK_COOL = 3
-- 1 - TEXT, 2 - STICKERS
local BDDTALLY_RANKS = {
	{"LOS", "LST"},
	{"WIN", "WST"},
	{"COOL", "CST"},
}

bdd.tally = {
	active = false,
	bonus = BDDTALLY_BONUS_NA,
	maxPositions = 0, -- this would just reference bdd.numPlayers
	oddLobby = 0, -- If the lobby is an odd size, account for it!
	-- animations
	state = 0,
	delay = 0,
	transition = 0,
	transitionTime = 0,
	lastTransition = 0,
	lines = 0,
	lineCount = 0,
	ticksnd = 0,
	showgrade = false,
}

local function BDD_TallyInit(self)
	self.active = true -- We are actually active now.
	-- Reset everything...
	self.bonus = bdd.duelmode and BDDTALLY_BONUS_TIMELEFT or BDDTALLY_BONUS_POSITION
	self.maxPositions = bdd.numplayers
	-- For odd size lobbies; add an extra 1 if size is odd
	if bdd.numplayers % 2 then
		self.oddLobby = 1
	else
		self.oddLobby = 0
	end
	
	for p in players.iterate do
		if p.spectator then continue end -- Fucking die spectators
		-- if isdedicatedserver and p == server then continue end -- And the server, too!
		-- Player specific
		if bdd.debuglogger.value then
			CONS_Printf(p, "\139BDD DEBUG:\128 MATCH OVER! Evaluating the player...")
			CONS_Printf(p, "\139BDD DEBUG:\128 Your position: " .. p.position)
		end
		p.bddTallyRank = p.position > ((self.maxPositions / 2) + self.oddLobby) and BDDTALLY_RANK_LOSE or (p.position == 1 and BDDTALLY_RANK_WINNER or BDDTALLY_RANK_COOL)
		if bdd.duelmode then -- enforce lose and winner on duel mode
			p.bddTallyRank = p.position > 1 and BDDTALLY_RANK_LOSE or BDDTALLY_RANK_WINNER
		end
		p.bddDisplayTime = 0
		p.bddDisplayStat = 0
		if bdd.debuglogger.value then
			CONS_Printf(p, "\139BDD DEBUG:\128 Therefore, your rank is " .. p.bddTallyRank)
		end
	end
	
	-- Start the state machine
	self.delay = 3*TICRATE
	self.state = BDDTALLY_STATE_GOTTHRU_SLIDEIN
	self.transition = 0
	self.transitionTime = TICRATE/2
	self.lastTransition = 0
	self.lineCount = 2 -- always two! 
	self.endonnext = false
end
rawset(_G, "BDD_TallyInit", BDD_TallyInit)

local function BDD_TallyNewLine(self)
	self.state = BDDTALLY_STATE_TEXT_APPEAR
	self.lines = self.lineCount
	self.delay = TICRATE/5
end

local function BDD_TallyILine(self)
	self.lines = $ - 1
	return true
end 

local function BDD_TallyTick(self)
	if not self.active then return end
	
	self.ticksnd = max(0, $-1)
	if self.delay > 0 then
		self.delay = $-1
		return
	end
	self.lastTransition = self.transition
	if self.transition < FRACUNIT then
		if self.transitionTime <= 0 then
			self.transition = FRACUNIT
			return
		end
		self.transition = min(FRACUNIT, $ + (FRACUNIT/self.transitionTime))
		return
	end
	
	if self.state == BDDTALLY_STATE_GOTTHRU_SLIDEIN then
		self.state = BDDTALLY_STATE_GOTTHRU_SLIDEUP
		self.delay = TICRATE/2
		self.transition = 0
		self.transitionTime = TICRATE/2
	elseif self.state == BDDTALLY_STATE_GOTTHRU_SLIDEUP then
		self.state = BDDTALLY_STATE_BOXES_SLIDEIN
		self.delay = TICRATE/5
		self.transition = 0
		self.transitionTime = TICRATE/2
	elseif self.state == BDDTALLY_STATE_BOXES_SLIDEIN then
		BDD_TallyNewLine(self)
	elseif self.state == BDDTALLY_STATE_TEXT_APPEAR then
		S_StopSoundByID(nil, sfx_mbs5b)
		S_StartSound(nil, self.endonnext and sfx_mbs70 or sfx_mbs5b)
		self.state = BDDTALLY_STATE_TEXT_PAUSE
		self.delay = TICRATE/2
	elseif self.state == BDDTALLY_STATE_TEXT_PAUSE then
		if not self.endonnext then
			self.state = BDDTALLY_STATE_TEXT_APPEAR
			self.delay = TICRATE/5
			self.endonnext = true
		else
			self.state = BDDTALLY_STATE_RANK_APPEAR
			self.delay = TICRATE/2
			self.lastTransition = 0
			self.transition = 0
			self.transitionTime = TICRATE
		end
		self.endonnext = true
	elseif self.state == BDDTALLY_STATE_RANK_APPEAR then
		S_StartSound(nil, sfx_rank)
		self.state = BDDTALLY_STATE_RANK_PAUSE
		self.delay = TICRATE/2
	elseif self.state == BDDTALLY_STATE_RANK_PAUSE then
		self.state = BDDTALLY_STATE_DONE
		self.delay = TICRATE*5
	elseif self.state == BDDTALLY_STATE_DONE then
		-- nothing...
	end
end
rawset(_G, "BDD_TallyTick", BDD_TallyTick)

local function intermissiontitledraw(v, x, y, header)
	v.drawScaled(20*FRACUNIT + x, 24*FRACUNIT + y, FRACUNIT, v.cachePatch("R_RTPBR"), 0)
	customhud.CustomFontString(v, x + (50<<FRACBITS), y + (7<<FRACBITS), header, "GTOL", 0, "left", FRACUNIT, 0)
	customhud.CustomFontString(v, x + (50<<FRACBITS), y + (7<<FRACBITS), header, "GTFN", 0, "left", FRACUNIT, 0)
	v.drawScaled(50*FRACUNIT + x, 42*FRACUNIT + y, FRACUNIT, v.cachePatch("R_GTHRO"), 0)
end

local function BDD_TallyDraw(self, v, p, c)
	-- Fuck you. No splitscreen.
	if v.splitscreen and v.splitscreen() > 0 then return end
	if not self.active then return end
	if p.spectator then return end
	
	local v_width = 320
	local v_height = 200
	
	local transition_i = FRACUNIT - self.transition
	
	local header_width = 320
	local header_x = (v_width - header_width) / 2

	local header_height = 36
	local header_centered = (v_height/2) - header_height
	
	-- banner
	if self.state == BDDTALLY_STATE_GOTTHRU_SLIDEIN then
		intermissiontitledraw(v, (header_x * FRACUNIT) + (v_width * transition_i), header_centered*FRACUNIT, skins[p.skin].realname)
	elseif self.state == BDDTALLY_STATE_GOTTHRU_SLIDEUP then
		intermissiontitledraw(v, (header_x * FRACUNIT), header_centered*transition_i, skins[p.skin].realname)
	else
		intermissiontitledraw(v, (header_x * FRACUNIT), 0, skins[p.skin].realname)
	end
	
	-- boxes.
	if self.state <= BDDTALLY_STATE_GOTTHRU_SLIDEUP then return end
	
	local bg = v.cachePatch("RNKBLK1A")
	local fg = v.cachePatch("RNKBLK1B")
	local sticker = v.cachePatch("K_STTIME")
	local offs = 3*FRACUNIT
	local drawer_x, drawer_y = ((v_width / 2) - (bg.width/2))<<FRACBITS, (((v_height / 2)+8) - ((bg.height/2)*self.lineCount))<<FRACBITS
	local olddrawer_x = drawer_x
	
	-- Here it comes.... the TC_CMAPHACK !!! WOOHOO !!!!
	local TC_CMAPHACK = v.splitscreen and TC_DEFAULT or 0
	local cmap = v.getColormap(TC_CMAPHACK, SKINCOLOR_BLUE)
	local textcmap = bdd.maintimercolor
	local boxflags = V_ADD
	
	for i=1,self.lineCount do
		local b = i-1
		if self.state == BDDTALLY_STATE_BOXES_SLIDEIN then
			drawer_x = $ + ((b&1) and 1 or -1) * FixedMul(transition_i, v_width*FRACUNIT)
		end
		v.drawScaled(drawer_x, drawer_y, FRACUNIT, fg, boxflags, cmap)
		v.drawScaled(drawer_x, drawer_y, FRACUNIT, bg, 0)
		if self.state <= BDDTALLY_STATE_BOXES_SLIDEIN then drawer_y = $ + (bg.height<<FRACBITS); drawer_x = olddrawer_x; continue end
		-- draw stats :))))
		local textx, texty = drawer_x + 11*FRACUNIT, drawer_y + 6*FRACUNIT
		texty = $ + (17*FRACUNIT)/2
		if b == 0 then
			-- draw time .... grrr
			v.drawScaled(textx+100*FRACUNIT, texty-2*FRACUNIT, FRACUNIT, v.cachePatch("EGGSTKR"), 0)
			v.drawScaled(textx, texty-1*FRACUNIT, FRACUNIT, v.cachePatch("BNS1P_TM"), 0)
			v.drawScaled(textx+(197*FRACUNIT)-((sticker.width/2)*FRACUNIT), texty-offs, FRACUNIT, sticker, 0)
			-- Timer for how long the match has lasted
			local timemin = G_TicsToMinutes(p.bddDeathTime)
			local timesec = G_TicsToSeconds(p.bddDeathTime)
			local timems = G_TicsToMilliseconds(p.bddDeathTime) / 10
			-- Draw the actual timer
			local curtime = string.format("%02d\'%02d\"%02d", timemin, timesec, timems) 
			-- " -- slade is fucking stupid
			-- strokin on mah d till i custom string drawer
			customhud.CustomFontString(v, textx+(197*FRACUNIT), texty, curtime, "BDFNT", 0, "center", FRACUNIT, textcmap)
		end
		-- draw the bonus !!!!
		if self.endonnext and b == 1 then
			local pts = bdd.duelmode and p.bdddueltime or p.bddLastPosition
			local over = bdd.duelmode and 0 or self.maxPositions
			if bdd.duelmode then over = 0 end
			local gfx = v.cachePatch(bdd.duelmode and "BDD_TLP1" or "BDD_PSP1")
			v.drawScaled(textx+100*FRACUNIT, texty-2*FRACUNIT, FRACUNIT, v.cachePatch("EGGSTKR"), 0)
			v.drawScaled(textx, texty-1*FRACUNIT, FRACUNIT, gfx, 0)
			v.drawScaled(textx+(197*FRACUNIT)-((sticker.width/2)*FRACUNIT), texty-offs, FRACUNIT, sticker, 0)
			-- Timer for how long the duel duel time is
			local timemin = G_TicsToMinutes(p.bdddueltime)
			local timesec = G_TicsToSeconds(p.bdddueltime)
			local timems = G_TicsToMilliseconds(p.bdddueltime) / 10
			-- Draw the actual timer
			local curtime = string.format("%02d\'%02d\"%02d", timemin, timesec, timems) 
			-- " -- slade is fucking stupid 2
			local fstr = over ~= 0 and string.format("%d/%d", pts, over) or curtime
			-- strokin on mah d till i custom string drawer
			customhud.CustomFontString(v, textx+(197*FRACUNIT), texty, fstr, "BDFNT", 0, "center", FRACUNIT, textcmap)
		end
		drawer_y = $ + (bg.height<<FRACBITS)
		drawer_x = olddrawer_x
	end
	
	if self.state < BDDTALLY_STATE_RANK_APPEAR then return end
	
	-- ARE YOU A WINNER!! OR A LOSER...
	local prefix = DBU_UpdateSplitscreen(v) > 1 and "BD4_" or "BDD_"
	local winner = v.cachePatch(prefix..BDDTALLY_RANKS[p.bddTallyRank][1])
	local code = BDDTALLY_RANKS[p.bddTallyRank][2]
	local stickerleft = v.cachePatch(prefix..code.."L")
	local stickerright = v.cachePatch(prefix..code.."R")
	
	-- centerit
	local sx, sy = ((v_width / 2) - (winner.width/2))<<FRACBITS, drawer_y + 10*FRACUNIT
	local smth = ease.outsine(self.transition, FRACUNIT, 0)
	local leftx = ((v_width / 2)<<FRACBITS) - ((winner.width/2)<<FRACBITS) - ((v_width / 2)*smth)
	local rightx = ((v_width / 2)<<FRACBITS) + ((winner.width/2)<<FRACBITS) + ((v_width / 2)*smth)
	local transy = sy + (200 * smth)
	
	-- real
	v.drawScaled(sx, transy, FRACUNIT, winner, 0)
	v.drawScaled(leftx, sy, FRACUNIT, stickerleft, 0)
	v.drawScaled(rightx, sy, FRACUNIT, stickerright, 0)
end

hud.add(function(v, p, c)
	if gametype ~= GT_BDD then return end
	if not bdd.gameover then return end
	BDD_TallyDraw(bdd.tally, v, p, c)
end, "game")

-- Heh.
local excludeArchive = { -- Archiving is forbidden
	["function"] = true, 
	["thread"] = true, 
	["proto"] = true, 
	["upval"] = true
}
local allowUDArchive = { -- Archiving is allowed :)
	["mobjinfo_t"] = true, 
	["state_t"] = true, 
	["mobj_t"] = true, 
	["player_t"] = true, 
	["mapthing_t"] = true, 
	["vertex_t"] = true, 
	["line_t"] = true, 
	["side_t"] = true, 
	["subsector_t"] = true, 
	["sector_t"] = true, 
	["ffloor_t"] = true, 
	["polyobj_t"] = true, 
	["slope_t"] = true, 
	["mapheader_t"] = true, 
	["skincolor_t"] = true, 
}
local function checksanity(t, net)
	for k, v in ipairs(t) do
		-- Table, recursively check
		if type(v) == "table" then checksanity(v, net) end
		-- Excluded from synching and potentially losing causing a sync failure.
		if excludeArchive[type(v)] then continue end 
		-- Ditto.
		if type(v) == "userdata" and (not allowUDArchive[userdataType(v)]) then continue end
		v = net($)
	end
end
-- Hopefully this doesn't cause any bad issues...
addHook("NetVars", function(net)
	checksanity(bdd.tally, net)
end)
