-- derbyUtils.lua
-- A series of helper scripts for various related things. Mostly splitscreen shit.
--
-- Made by haya. This is reusable. Feel free to mangle as much as you want.
-- Will be updated with more shit as time goes on.
-- ...Or not.

-- 2.4/HEP3 check, checks for the followers array's existence.
local CACHED_VCHECK = followers ~= nil

-- DBU_HUDToggleOffset

local HUD_OFFS = {
	["names"] = "check",
	["check"] = "minirankings",
	["minirankings"] = "battlerankingsbumpers",
	["battlerankingsbumpers"] = "wanted",
	["wanted"] = "speedometer",
	["speedometer"] = "freeplay",
	["rankings"] = "intermissiontally",
	["intermissiontally"] = "intermissionmessages",
	-- the real intermissionmessages is completely inaccessible.
}

-- DBU_HUDToggleOffset(item)
-- Takes in an item, and either returns that item, or an offsetted item element
-- to compensate for 2.3's hud item tables.
--
-- item - The (intended) item to return.
local function DBU_HUDToggleOffset(item)
	-- No need to do anything.
	if CACHED_VCHECK then return item end
	return HUD_OFFS[item] and HUD_OFFS[item] or item
end
rawset(_G, "DBU_HUDToggleOffset", DBU_HUDToggleOffset)

-- DBU_*SplitScreen

-- DBU_FSplitscreen
-- Stands for FALSE splitscreen.
rawset(_G, "DBU_FSplitscreen", 0)

-- DBU_UpdateSplitscreen(v)
-- Updates the splitscreen variable. Requires GHG's fixdisplayplayers.lua script.
-- Also returns the current splitscreen variable.
--
-- v - Drawer object to return splitscreen on 2.4+
local function DBU_UpdateSplitscreen(v)
	-- 2.4+ / HEP3
	-- This will be the one used going forward.
	if v.splitscreen then
		if CACHED_VCHECK then DBU_FSplitscreen = v.splitscreen(); return DBU_FSplitscreen; end
	end
	-- 2.3 or below
	-- Use displayplayers hack....
	-- thanks genericheroguy
	DBU_FSplitscreen = 0
	for dp in displayplayers.iterate do
		DBU_FSplitscreen = $ + 1
	end
	DBU_FSplitscreen = $ - 1
	return DBU_FSplitscreen
end
rawset(_G, "DBU_UpdateSplitscreen", DBU_UpdateSplitscreen)

-- DBU_SnapSplitscreen(v, c, x, y, scr)
-- Handle splitscreen positioning without V_SPLITSCREEN.
-- Requires draw calls using the returned coordinates to 
-- always have V_SNAPTOTOP and V_SNAPTOLEFT as flags.
--
-- v - Drawer
-- c - Camera
-- x - X coordinate of what you're drawing in fracunits
-- y - Y coordinate of what you're drawing in fracunits
-- scr - Additional snap flags
-- spl - Optional splitscreen variable to use over DBU_FSplitscreen.
--
-- Returns the edited X and Y coords in fixed point.
local function DBU_SnapSplitscreen(v, c, x, y, scr, spl)
	scr = $ or 0
	spl = $ or DBU_FSplitscreen
	-- heeere it is
	local screenwidth = (v.width()/v.dupx())<<FRACBITS
	local screenheight = (v.height()/v.dupy())<<FRACBITS
	local basewidth = 320<<FRACBITS
	local baseheight = 200<<FRACBITS
	local splitnum = c.pnum - 1 -- can you believe it guys??? an undocumented feature !!!!
	-- If splitscreen is not available, then simply default to 0
	-- TODO: Use a displayplayers hack to do normal splitscreen on 2.3
	local splitscr = spl
	
	-- Handle halving depending on splitscreen shit
	if splitscr > 0 then
		screenheight = $ / 2
		baseheight = $ / 2
		if splitscr > 1 then 
			screenwidth = $ / 2
			basewidth = $ / 2
		end
	end
	
	-- Splitscreen non 16:10 resolution never works. Lets try anyway.
	if v.width() ~= 320*v.dupx() then
		if (scr & V_SNAPTORIGHT) then x = $ + (screenwidth - basewidth) 
		elseif ((scr & V_SNAPTOLEFT) ~= V_SNAPTOLEFT) then x = $ + ((screenwidth - basewidth) / 2) end
	end
	
	if v.height() ~= 200*v.dupy() then
		if (scr & V_SNAPTOBOTTOM) then y = $ + (screenheight - baseheight) 
		elseif ((scr & V_SNAPTOTOP) ~= V_SNAPTOTOP) then y = $ + ((screenheight - baseheight) / 2) end
	end
	
	-- The part thats actually important
	if splitscr == 1 and splitnum == 1 then
		y = $ + screenheight
	elseif splitscr > 1 then
		if (splitnum == 1 or splitnum == 3) then
			x = $ + screenwidth
		end
		if (splitnum >= 2) then
			y = $ + screenheight
		end
	end
	
	return x, y
end
rawset(_G, "DBU_SnapSplitscreen", DBU_SnapSplitscreen)
-- Alt name because this was the original name teehee
rawset(_G, "DBU_SplitscreenFUCK", DBU_SnapSplitscreen)

-- DBU_TallyHallOffset(v, scr)
-- Returns a V_SLIDEIN style offset for various purposes.
--
-- v - Drawer
-- timer - The timer to base the slide upon. Required.
-- scr - Snap flags
-- len - Length of the slide. Defaults to TICRATE/4.
--
-- Returns an offset in fixed point.
local function DBU_TallyHallOffset(v, timer, scr, len)
	if timer == 0 then return 0 end
	
	local length = len or TICRATE/4
	local timer = length - timer
	local f = timer * FRACUNIT
	-- once again, no interp
	local st_fadein = f / length
	
	-- no splitscreen support for this one
	local screenwidth = v.width()/v.dupx()
	local screenheight = v.height()/v.dupy()
	
	if st_fadein >= FRACUNIT then return (screenwidth * FRACUNIT/2) end

	-- left and right snap
	if (scr & (V_SNAPTOBOTTOM|V_SNAPTOTOP)) then
		local slidefromright = (scr & V_SNAPTORIGHT)
		local offsetAmt = screenwidth * FRACUNIT/2
		local offset = offsetAmt - FixedMul(offsetAmt, st_fadein)
		
		return slidefromright and offset or -offset
	end
	
	-- top and bottom snap
	local offsetAmt = screenheight * FRACUNIT/2
	local offset = offsetAmt - FixedMul(offsetAmt, st_fadein)
		
	return (scr & V_SNAPTOBOTTOM) and offset or -offset
end
rawset(_G, "DBU_TallyHallOffset", DBU_TallyHallOffset)

-- DBU_FuckerIsValid(p)
-- Returns if a player is valid AND in-game.
--
-- p - The player to check.
--
-- Returns true if valid, false if not.
local function DBU_FuckerIsValid(p)
	-- Of course, check for nil and p.valid first
	if not (p and p.valid) then return false end
	-- Check for spectator...
	if p.spectator then return false end
	-- Check if mo is valid. Yes, dedicated servers skip the spectator check.
	if not (p.mo and p.mo.valid) then return false end
	
	-- Otherwise, it is a valid player.
	return true
end
rawset(_G, "DBU_FuckerIsValid", DBU_FuckerIsValid)
